require 'spec_helper'

describe SubZero::Client do

  subject { described_class.new :sid }

  let(:config) { { ip: '127.0.0.1', port: 7777 } }

  its(:sid)    { should eq('SID') }
  its(:config) { should eq(config) }

  describe '#socket' do
    let(:socket) { double :socket }

    before do
      SubZero::Socket.stub(:new).with(config) { socket }
    end

    its(:socket) { should eq(socket) }
  end

  describe '#call' do
    let(:socket)   { double :socket }
    let(:response) { double :response, error?: false, payload: { found: true } }

    before do
      subject.stub(:socket) { socket }

      socket.should_receive :call do |request|
        request.type.should eq('REQ')
        request.sid.should  eq('SID')
        request.verb.should eq('GET/USER')
        request.payload.should eq(id: 2)
        request.options.should eq(timeout: 30)
        response
      end
    end

    it 'returns response payload' do
      subject.call(:get_user, { id: 2 }, { timeout: 30 }).should eq(found: true)
    end

    it 'injects full response in block' do
      subject.call :get_user, { id: 2 }, { timeout: 30 } do |result|
        result.should eq(response)
      end
    end
  end

  describe '#[]' do
    let(:response) { double :response }

    it 'is a different way to do call' do
      subject.should_receive(:call).with(:get_user) { response }
      subject[:get_user].should eq(response)
    end
  end

  describe 'calling verbs via method missing' do
    let(:response) { double :response }
    let(:args)     { [{ id: 2 }, { timeout: 3000 }] }

    it 'calls verb with method missing name' do
      subject.should_receive(:call).with(:get_user, *args) { response }
      subject.get_user(*args).should eq(response)

      subject.should_receive(:call!).with('get_user', *args) { response }
      subject.get_user!(*args).should eq(response)
    end
  end

end
